"""A getter for HGNC."""

import datetime
import logging

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "HGNCGetter",
]

logger = logging.getLogger(__name__)

PATH = "https://storage.googleapis.com/public-download-files/hgnc/archive/archive/monthly/json/"
PREFIX = "hgnc_complete_set_"
SUFFIX = ".json"


class HGNCGetter(Getter):
    """A getter for HGNC."""

    bioregistry_id = "hgnc"
    name = "HGNC"
    homepage_fmt = PATH + "hgnc_complete_set_{version}.json"

    version_type = VersionType.date

    def get(self) -> str:
        """Get the latest HGNC version number."""
        today = datetime.date.today()
        this_year = int(today.strftime("%Y"))
        this_month = int(today.strftime("%m"))
        maybe = today.strftime("%Y-%m-01")
        res = requests.head(self.homepage_fmt.format(version=maybe))
        if res.status_code == 200:
            return maybe
        if this_month == 1:
            maybe_last_month = f"{this_year - 1}-12-01"
        else:
            maybe_last_month = f"{this_year}-{this_month - 1}-01"
        return maybe_last_month


if __name__ == "__main__":
    HGNCGetter.print()



# -*- coding: utf-8 -*-

"""A getter for UniProt."""

from xml.etree import ElementTree  # noqa:S405

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "UniProtGetter",
]


class UniProtGetter(Getter):
    """A getter for UniProt."""

    bioregistry_id = "uniprot"
    name = "UniProt"
    homepage_fmt = (
        "ftp://ftp.uniprot.org/pub/databases/uniprot/previous_releases/release-{version}/"
    )
    date_version_fmt = "%Y_%m"
    version_type = VersionType.month

    def get(self):
        """Get the latest UniProt version number."""
        session = requests.Session()
        f = session.get(
            "https://ftp.uniprot.org/pub/databases/uniprot/current_release/RELEASE.metalink"
        )
        tree = ElementTree.fromstring(f.text)  # noqa:S314
        version_tag = tree.find("{http://www.metalinker.org/}version")
        return version_tag.text


if __name__ == "__main__":
    UniProtGetter.print()



"""Get the version for CiVIC."""

import requests

from bioversions.utils import Getter, VersionType

URL = "https://civicdb.org/releases/main"
API = "https://civicdb.org/api/graphql"
GRAPHQL_QUERY = """\
query dataReleases {
  dataReleases {
    geneTsv {
      filename
      path
    }
    name
  }
}
"""
# see https://griffithlab.github.io/civic-v2/#query-dataReleases
# and https://griffithlab.github.io/civic-v2/#definition-DownloadableFile


class CiVICGetter(Getter):
    """A getter for CiVIC."""

    name = "CiVIC"
    date_fmt = "%d-%b-%Y"
    version_type = VersionType.date
    homepage = "https://civicdb.org"
    collection = ["civic.gid", "civic.eid"]

    def get(self):
        """Get the latest ChEMBL version number."""
        res = requests.post(API, json={"query": GRAPHQL_QUERY})
        # 0 element is always nightly, 1 is latest
        return res.json()["data"]["dataReleases"][1]["name"]


if __name__ == "__main__":
    CiVICGetter.print()



# -*- coding: utf-8 -*-

"""A getter for PomBase."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "PombaseGetter",
]


class PombaseGetter(Getter):
    """A getter for PomBase."""

    bioregistry_id = "pombase"
    name = "PomBase"
    date_version_fmt = "%Y-%m-%d"
    homepage_fmt = "https://www.pombase.org/data/releases/pombase-{version}/"
    version_type = VersionType.date

    def get(self):
        """Get the latest pombase version number."""
        soup = get_soup("https://www.pombase.org/data/releases/")
        tr = soup.find_all("tr")[-2]
        text = tr.find_all("td")[1]
        anchor = text.find("a")
        text = anchor.text
        return text[len("pombase-") :].rstrip("/")


if __name__ == "__main__":
    PombaseGetter.print()



# -*- coding: utf-8 -*-

"""A getter for Rhea."""

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "RheaGetter",
]

VERSION_FILE = "https://ftp.expasy.org/databases/rhea/rhea-release.properties"


class RheaGetter(Getter):
    """A getter for Rhea."""

    bioregistry_id = "rhea"
    name = "Rhea"
    date_fmt = "%Y-%m-%d"
    version_type = VersionType.date

    def get(self):
        """Get the latest Rhea version number."""
        with requests.Session() as session:
            res = session.get(VERSION_FILE)
            d = dict(line.strip().split("=") for line in res.text.splitlines() if line.strip())
        return {
            "version": d["rhea.release.number"],
            "date": d["rhea.release.date"].title(),
        }


if __name__ == "__main__":
    RheaGetter.print()



# -*- coding: utf-8 -*-

"""A getter for Reactome."""

from ..utils import Getter, VersionType, get_soup

__all__ = [
    "ReactomeGetter",
]

URL = "https://reactome.org/"


class ReactomeGetter(Getter):
    """A getter for Reactome."""

    bioregistry_id = "reactome"
    name = "Reactome"
    version_type = VersionType.sequential

    def get(self):
        """Get the latest BioGRID version number."""
        soup = get_soup(URL)
        manifest = soup.find(id="fav-portfoliowrap")
        header = manifest.find("h3")
        return header.text.split()[1]


if __name__ == "__main__":
    ReactomeGetter.print()
